import os
import sys
import unittest
import nose.plugins
from warnings import warn

from nose.config import Config
from nose.plugins.attrib import AttributeSelector
from nose.plugins.base import Plugin
from nose.plugins.cover import Coverage
from nose.plugins.doctests import Doctest
from nose.plugins.missed import MissedTests
from nose.plugins.profile import Profile

from mock import *

class P(Plugin):
    """Plugin of destiny!"""    
    pass

# some plugins have 2.4-only features
pyvers = sys.version_info
compat_24 = pyvers[0] >= 2 and pyvers[1] >= 4

class TestBuiltinPlugins(unittest.TestCase):

    def setUp(self):
        self.p = sys.path[:]

    def tearDown(self):
        sys.path = self.p[:]
    
    def test_load(self):
        plugs = list(nose.plugins.load_plugins(builtin=True, others=False))
        # print plugs

        assert Coverage in plugs
        assert Doctest in plugs
        assert AttributeSelector in plugs
        assert Profile in plugs
        assert MissedTests in plugs
        assert len(plugs) == 5

        for p in plugs:
            assert not p.enabled
            
    def test_add_options(self):
        conf = Config()
        opt = Bucket()
        parser = MockOptParser()
        plug = P()

        plug.add_options(parser)
        o, d = parser.opts[0]
        # print d
        assert o[0] == '--with-p'
        assert d['action'] == 'store_true'
        assert not d['default']
        assert d['dest'] == 'enable_plugin_p'
        assert d['help'] == 'Enable plugin P: Plugin of destiny! [NOSE_WITH_P]'

        opt.enable_plugin_p = True
        plug.configure(opt, conf)
        assert plug.enabled

    def test_tolist(self):
        plug = P()
        assert plug.tolist('foo') == ['foo']
        assert plug.tolist(['foo', 'bar']) == ['foo', 'bar']
        assert plug.tolist('foo,bar') == ['foo', 'bar']
        self.assertEqual(plug.tolist('.*foo/.*,.1'), ['.*foo/.*', '.1'])
        
class TestDoctestPlugin(unittest.TestCase):

    def setUp(self):
        self.p = sys.path[:]

    def tearDown(self):
        sys.path = self.p[:]
    
    def test_add_options(self):
        # doctest plugin adds some options...
        conf = Config()
        opt = Bucket()
        parser = MockOptParser()
        plug = Doctest()
        
        plug.add_options(parser, {})
        o, d = parser.opts[0]
        assert o[0] == '--with-doctest'

        o2, d2 = parser.opts[1]
        assert o2[0] == '--doctest-tests'

        if compat_24:
            o3, d3 = parser.opts[2]
            assert o3[0] == '--doctest-extension'
        else:
            assert len(parser.opts) == 2
    
    def test_want_file(self):
        # doctest plugin can select module and/or non-module files
        conf = Config()
        opt = Bucket()
        plug = Doctest()
        plug.configure(opt, conf)
        
        assert plug.wantFile('foo.py') is None
        assert not plug.wantFile('bar.txt')
        assert not plug.wantFile('buz.rst')
        assert not plug.wantFile('bing.mov')
        
        plug.extension = ['.txt', '.rst']
        assert plug.wantFile('/path/to/foo.py') is None
        assert plug.wantFile('/path/to/bar.txt')
        assert plug.wantFile('/path/to/buz.rst')
        assert not plug.wantFile('/path/to/bing.mov')
        
    def test_matches(self):
        # doctest plugin wants tests from all NON-test modules
        conf = Config()
        opt = Bucket()
        plug = Doctest()
        plug.configure(opt, conf)
        assert not plug.matches('test')
        assert plug.matches('foo')

    def test_collect_pymodule(self):
        here = os.path.dirname(__file__)
        support = os.path.join(here, 'support')
        if not support in sys.path:
            sys.path.insert(0, support)
        import foo.bar.buz
        
        conf = Config()
        opt = Bucket()
        plug = Doctest()
        plug.configure(opt, conf)
        suite = plug.loadTestsFromModule(foo.bar.buz)
        if compat_24:
            expect = ['afunc (foo.bar.buz)']
        else:
            expect = ['unittest.FunctionTestCase (runit)']            
        for test in suite:
            self.assertEqual(str(test), expect.pop(0))
            
    def test_collect_txtfile(self):
        if not compat_24:
            warn("No support for doctests in files other than python modules"
                 " in python versions older than 2.4")
            return
        here = os.path.abspath(os.path.dirname(__file__))
        support = os.path.join(here, 'support')
        fn = os.path.join(support, 'foo', 'doctests.txt')
        
        conf = Config()        
        opt = Bucket()
        plug = Doctest()
        plug.configure(opt, conf)
        plug.extension = ['.txt']
        suite = plug.loadTestsFromPath(fn)
        for test in suite:
            assert str(test).endswith('doctests.txt')

class TestAttribPlugin(unittest.TestCase):

    def test_add_options(self):
        plug = AttributeSelector()
        parser = MockOptParser()
        plug.add_options(parser)

        expect = [(('-a', '--attr'),
                   {'dest': 'attr', 'action': 'append', 'default': None,
                    'help': 'Run only tests that have attributes '
                    'specified by ATTR [NOSE_ATTR]'})]

        if compat_24:
            expect.append(
                (('-A', '--eval-attr'),
                 {'dest': 'eval_attr', 'action': 'append',
                  'default': None, 'metavar': 'EXPR',
                  'help': 'Run only tests for whose attributes the '
                  'Python expression EXPR evaluates to True '
                  '[NOSE_EVAL_ATTR]'}))
        self.assertEqual(parser.opts, expect)

        opt = Bucket()
        opt.attr = ['!slow']
        plug.configure(opt, Config())
        assert plug.enabled
        self.assertEqual(plug.attribs, [('slow',False)])

        opt.attr = ['fast,quick', 'weird=66']
        plug.configure(opt, Config())
        self.assertEqual(plug.attribs, [('fast', True),
                                        ('quick', True),
                                        ('weird', '66')])

        if compat_24:
            opt.attr = None
            opt.eval_attr = [ 'weird >= 66' ]
            plug.configure(opt, Config())
            self.assertEqual(plug.attribs[0][0], 'weird >= 66')
            assert callable(plug.attribs[0][1])
                       
    def test_basic_attr(self):
        def f():
            pass
        def g():
            pass
    
        plug = AttributeSelector()
        f.a = 1
        plug.attribs = [('a',1)]
        assert plug.wantFunction(f) is not False
        assert not plug.wantFunction(g)

    def test_eval_attr(self):
        if not compat_24:
            warn("No support for eval attributes in python versions older"
                 " than 2.4")
            return
        def f():
            pass
        f.monkey = 2
        
        def g():
            pass
        g.monkey = 6

        def h():
            pass
        h.monkey = 5
        
        cnf = Config()
        opt = Bucket()
        opt.eval_attr = "monkey > 5"
        plug = AttributeSelector()
        plug.configure(opt, cnf)

        assert not plug.wantFunction(f)
        assert plug.wantFunction(g) is not False
        assert not plug.wantFunction(h)

        
class TestMissedTestsPlugin(unittest.TestCase):

    def test_options(self):
        pass

    def test_match(self):
        class FooTest(unittest.TestCase):
            def test_bar(self):
                pass
            
        class QuzTest:
            def test_whatever(self):
                pass
        def test_baz():
            pass
        foo = FooTest('test_bar')
        baz = nose.case.FunctionTestCase(test_baz)
        quz = nose.case.MethodTestCase(QuzTest, 'test_whatever')
        
        plug = MissedTests()

        here = os.path.abspath(__file__)
        
        # positive matches
        assert plug.match(foo, ':FooTest.test_bar')
        assert plug.match(foo, ':FooTest')
        assert plug.match(foo, here)
        assert plug.match(foo, os.path.dirname(here))
        assert plug.match(foo, __name__)
        assert plug.match(baz, ':test_baz')
        assert plug.match(baz, here)
        assert plug.match(baz, __name__)
        assert plug.match(quz, ':QuzTest.test_whatever')
        assert plug.match(quz, ':QuzTest')
        assert plug.match(quz, here)
        assert plug.match(quz, __name__)
        
        # non-matches
        assert not plug.match(foo, ':test_bar')        
        assert not plug.match(foo, ':FooTest.test_bart')
        assert not plug.match(foo, 'some.module')
        assert not plug.match(foo, __name__ + '.whatever')
        assert not plug.match(foo, '/some/path')
        
if __name__ == '__main__':
    unittest.main()
